@font-face {
  font-family: "lobsterregular";
  src: url("./fonts/lobster-regular-webfont.woff2") format("woff2"),
    url("./fonts/lobster-regular-webfont.woff") format("woff");
  font-weight: bold;
  font-style: normal;
}

@font-face {
  font-family: "work_sansregular";
  src: url("./fonts/worksans-variablefont_wght-webfont.woff2") format("woff2"),
    url("./fonts/worksans-variablefont_wght-webfont.woff") format("woff");
  font-weight: normal;
  font-style: normal;
}

* {
  font-family: work_sansregular;
}

.navbar-text {
  color: rgb(250, 250, 250);
}

.navbar-brand {
  color: rgb(250, 250, 250);
  font-family: lobsterregular;
  font-size: 1.5rem;
}

.navbar-brand:hover {
  color: rgb(250, 250, 250);
  font-family: lobsterregular;
}

.site-header-logo h1 {
  color: rgb(39, 147, 248);
  font-family: lobsterregular;
  font-size: 3rem;
  margin-top: 2rem;
}